<?php

namespace Drupal\skpr_key\Plugin\KeyProvider;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\key\KeyInterface;
use Drupal\key\Plugin\KeyProviderBase;
use Drupal\key\Plugin\KeyPluginFormInterface;

/**
 * Adds a key provider that allows a key to be loaded from Skpr config.
 *
 * @KeyProvider(
 *   id = "skpr",
 *   label = "Skpr",
 *   description = @Translation("This provider retrieves the key from Skpr config."),
 *   storage_method = "skpr",
 *   key_value = {
 *     "accepted" = FALSE,
 *     "required" = FALSE
 *   }
 * )
 */
class SkprKeyProvider extends KeyProviderBase implements KeyPluginFormInterface {

  /**
   * The Skpr config object.
   *
   * @var \Skpr\SkprConfig
   */
  protected $skpr;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    /** @var self $instance */
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->skpr = $container->get('skpr_key.skpr_config_factory')->getInstance();
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'skpr_key' => '',
      'fallback' => '',
      'base64_encoded' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    // Ensure keys always exist to reduce errors across version upgrades.
    return array_merge($this->defaultConfiguration(), parent::getConfiguration());
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $config = $this->getConfiguration();

    $options = [];
    foreach ($this->skpr->getAll() as $key => $value) {
      $options[$key] = $key;
    }

    $form['skpr_key'] = [
      '#type' => 'select',
      '#title' => $this->t('Skpr key'),
      '#description' => $this->t('Name of the skpr config to load.'),
      '#required' => TRUE,
      '#options' => $options,
      '#empty_value' => '',
      '#default_value' => $config['skpr_key'],
    ];

    $form['fallback'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Fallback'),
      '#description' => $this->t("Fallback value for when Skpr isn't available or the key isn't set. Useful for development.<br/><strong>Important: Only set this if a missing key will cause errors and do not set this to a real production value.</strong>"),
      '#empty_value' => '',
      '#default_value' => $config['fallback'],
    ];

    // If this key type is for an encryption key.
    if ($form_state->getFormObject()->getEntity()->getKeyType()->getPluginDefinition()['group'] == 'encryption') {
      // Add an option to indicate that the value is stored Base64-encoded.
      $form['base64_encoded'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Base64-encoded'),
        '#description' => $this->t('Check this if the key value is Base64-encoded.'),
        '#default_value' => $config['base64_encoded'],
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $key_provider_settings = $form_state->getValues();
    $skpr_key = $key_provider_settings['skpr_key'];

    // Does the config exist.
    $key_value = $this->skpr->get($skpr_key, '');
    if (empty($key_value)) {
      $form_state->setErrorByName('skpr_key', $this->t('The key does not exist or it is empty.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->setConfiguration($form_state->getValues());
  }

  /**
   * {@inheritdoc}
   */
  public function getKeyValue(KeyInterface $key) {
    $config = $this->getConfiguration();

    $key_value = $this->skpr->get($config['skpr_key'], $config['fallback']);
    if (!$key_value) {
      return NULL;
    }

    if ($config['base64_encoded'] == TRUE) {
      $key_value = base64_decode($key_value);
    }

    return $key_value;
  }

}
